@@ -7,10 +7,10 @@ require_once 'Horde/Identity.php';
  * user might have. Its methods take care of any site-specific
  * restrictions configured in prefs.php and conf.php.
  *
- * $Horde: imp/lib/Identity/imp.php,v 1.44.2.5 2005/07/03 03:42:44 selsky Exp $
+ * $Horde: imp/lib/Identity/imp.php,v 1.44.2.10 2007/01/02 13:54:58 jan Exp $
  *
- * Copyright 2001-2005 Jan Schneider <jan@horde.org>
- * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2001-2007 Jan Schneider <jan@horde.org>
+ * Copyright 2001-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -400,7 +400,7 @@ class Identity_imp extends Identity {
     }
 
     /**
-     * Returns the users full name.
+     * Returns the user's full name.
      *
      * @param integer $ident  The identity to retrieve the name from.
      *
@@ -427,8 +427,6 @@ class Identity_imp extends Identity {
      */
     function getSignature($ident = null)
     {
-        global $conf;
-
         if (isset($this->_signatures[$ident])) {
             return $this->_signatures[$ident];
         }
@@ -448,7 +446,7 @@ class Identity_imp extends Identity {
             }
         }
 
-        if (!empty($conf['hooks']['signature'])) {
+        if (!empty($GLOBALS['conf']['hooks']['signature'])) {
             include_once HORDE_BASE . '/config/hooks.php';
             if (function_exists('_imp_hook_signature')) {
                 $val = call_user_func('_imp_hook_signature', $val);
@@ -481,31 +479,14 @@ class Identity_imp extends Identity {
     }
 
     /**
-     * Returns the sent-mail folder name.
-     *
-     * @param integer $ident            The identity to retrive the folder name
-     *                                  from.
-     * @param string $sent_mail_folder  A default folder that is returned no
-     *                                  identity is selected and the
-     *                                  sent_mail_folder preference is not
-     *                                  locked.
-     *
-     * @return string  The folder name with the folder prefix.
+     * @see Identity::getValue()
      */
-    function getSentmailFolder($ident = null, $sent_mail_folder = '')
+    function getValue($key, $identity = null)
     {
-        if (!isset($ident)) {
-            $folder = $sent_mail_folder;
+        if ($key == 'sent_mail_folder') {
+            return IMP::folderPref(parent::getValue('sent_mail_folder', $identity), true);
         }
-        if (empty($folder) || $this->_prefs->isLocked('sent_mail_folder')) {
-            $folder = $this->getValue('sent_mail_folder', $ident);
-        }
-
-        if (substr($folder, 0, strlen(IMP::preambleString())) != IMP::preambleString()) {
-            $folder = IMP::addPreambleString($folder);
-        }
-
-        return $folder;
+        return parent::getValue($key, $identity);
     }
 
     /**
@@ -516,8 +497,11 @@ class Identity_imp extends Identity {
      */
     function getAllSentmailFolders()
     {
+        $list = array();
         foreach ($this->_identities as $key => $identity) {
-            $list[$this->getSentmailFolder($key)] = true;
+            if ($folder = $this->getValue('sent_mail_folder', $key)) {
+                $list[$folder] = true;
+            }
         }
         return array_keys($list);
     }
@@ -531,9 +515,7 @@ class Identity_imp extends Identity {
      */
     function saveSentmail($ident = null)
     {
-        global $conf;
-
-        if (!$conf['user']['allow_folders']) {
+        if (!$GLOBALS['conf']['user']['allow_folders']) {
             return false;
         }
 
