@@ -4,9 +4,9 @@
  * and provides an option to import the data into a calendar source,
  * if one is available.
  *
- * $Horde: imp/lib/MIME/Viewer/itip.php,v 1.37.2.17 2005/10/09 20:11:14 selsky Exp $
+ * $Horde: imp/lib/MIME/Viewer/itip.php,v 1.37.2.27 2007/03/06 15:25:33 jan Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -61,14 +61,15 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
 
         // Extract the data.
         $data = $this->mime_part->getContents();
-        $this->_getHeaders($params[0]);
+        if (empty($this->_headers) && is_a($params[0], 'IMP_Contents')) {
+            $this->_headers = $params[0]->getHeaderOb();
+        }
 
         // Parse the iCal file.
-        $vCal = &new Horde_iCalendar();
+        $vCal = new Horde_iCalendar();
         if (!$vCal->parsevCalendar($data)) {
-            $html = '<b>' . _("The calendar data is invalid") .
-                '</b><br /><pre>' . htmlspecialchars($data) . '</pre>';
-            return $html;
+            return '<h1>' . _("The calendar data is invalid") . '</h1>' .
+                '<pre>' . htmlspecialchars($data) . '</pre>';
         }
 
         // Check if we got vcard data with the wrong vcalendar mime
@@ -81,6 +82,9 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
 
         // Get the method type.
         $this->_method = $vCal->getAttribute('METHOD');
+        if (is_a($this->_method, 'PEAR_Error')) {
+            $this->_method = '';
+        }
 
         // Get the iCalendar file components.
         $components = $vCal->getComponents();
@@ -137,7 +141,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                         } else {
                             $url = Horde::url($registry->link('calendar/show', array('uid' => $guid)));
                             $this->_msgs[$key][] = array('success', _("The event was added to your calendar.") .
-                                                       '&nbsp;' . Horde::link($url, _("View event"), null, '_blank') . Horde::img('mime/icalendar.png', _("View event"), null, $registry->getImageDir('horde')) . '</a>');
+                                                         '&nbsp;' . Horde::link($url, _("View event"), null, '_blank') . Horde::img('mime/icalendar.png', _("View event"), null, $registry->getImageDir('horde')) . '</a>');
                         }
                     } else {
                         $this->_msgs[$key][] = array('warning', _("This action is not supported."));
@@ -167,7 +171,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                         } else {
                             $url = Horde::url($registry->link('tasks/show', array('uid' => $guid)));
                             $this->_msgs[$key][] = array('success', _("The task has been added to your tasklist.") .
-                                                       '&nbsp;' . Horde::link($url, _("View task"), null, '_blank') . Horde::img('mime/icalendar.png', _("View task"), null, $registry->getImageDir('horde')) . '</a>');
+                                                         '&nbsp;' . Horde::link($url, _("View task"), null, '_blank') . Horde::img('mime/icalendar.png', _("View task"), null, $registry->getImageDir('horde')) . '</a>');
                         }
                     } else {
                         $this->_msgs[$key][] = array('warning', _("This action is not supported."));
@@ -204,7 +208,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                     $organizerName = isset($organizer['cn']) ? $organizer['cn'] : '';
 
                     // Build the reply.
-                    $vCal = &new Horde_iCalendar();
+                    $vCal = new Horde_iCalendar();
                     $vCal->setAttribute('PRODID', '-//The Horde Project//' . HORDE_AGENT_HEADER . '//EN');
                     $vCal->setAttribute('METHOD', 'REPLY');
 
@@ -226,9 +230,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                         $vEvent_reply->setAttribute('DURATION', $vEvent->getAttribute('DURATION'), array_pop($duration));
                     }
                     if (!is_a($vEvent->getAttribute('SEQUENCE'), 'PEAR_error')) {
-                        $vEvent_reply->setAttribute('SEQUENCE', $vEvent->getAttribute('SEQUENCE') + 1);
-                    } else {
-                        $vEvent_reply->setAttribute('SEQUENCE', 0);
+                        $vEvent_reply->setAttribute('SEQUENCE', $vEvent->getAttribute('SEQUENCE'));
                     }
                     $organizer = $vEvent->getAttribute('ORGANIZER', true);
                     $vEvent_reply->setAttribute('ORGANIZER', $vEvent->getAttribute('ORGANIZER'), array_pop($organizer));
@@ -240,7 +242,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                         $attendees = array($attendees);
                     }
                     foreach ($attendees as $attendee) {
-                        $attendee = str_replace('MAILTO:', '', $attendee);
+                        $attendee = preg_replace('/mailto:/i', '', $attendee);
                         if (!is_null($id = $identity->getMatchingIdentity($attendee))) {
                             $identity->setDefault($id);
                             break;
@@ -271,24 +273,25 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                         break;
                     }
 
-                    $vEvent_reply->setAttribute('ATTENDEE', 'MAILTO:' . $email, $params);
+                    $vEvent_reply->setAttribute('ATTENDEE', 'mailto:' . $email, $params);
                     $vCal->addComponent($vEvent_reply);
 
-                    $mime = &new MIME_Part('multipart/alternative');
-                    $body = &new MIME_Part('text/plain',
-                                           String::wrap($message, 76, "\n"),
-                                           NLS::getCharset());
+                    $mime = new MIME_Part('multipart/alternative');
+                    $body = new MIME_Part('text/plain',
+                                          String::wrap($message, 76, "\n"),
+                                          NLS::getCharset());
 
-                    $ics = &new MIME_Part('text/calendar', $vCal->exportvCalendar());
+                    $ics = new MIME_Part('text/calendar', $vCal->exportvCalendar());
                     $ics->setName('event-reply.ics');
                     $ics->setContentTypeParameter('METHOD', 'REPLY');
+                    $ics->setCharset(NLS::getCharset());
 
                     $mime->addPart($body);
                     $mime->addPart($ics);
                     $mime = &MIME_Message::convertMimePart($mime);
 
                     // Build the reply headers.
-                    $msg_headers = &new MIME_Headers();
+                    $msg_headers = new MIME_Headers();
                     $msg_headers->addReceivedHeader();
                     $msg_headers->addMessageIdHeader();
                     $msg_headers->addHeader('Date', date('r'));
@@ -367,26 +370,27 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                     $organizerName = isset($organizer['cn']) ? $organizer['cn'] : '';
 
                     // Build the reply.
-                    $vCal = &new Horde_iCalendar();
+                    $vCal = new Horde_iCalendar();
                     $vCal->setAttribute('PRODID', '-//The Horde Project//' . HORDE_AGENT_HEADER . '//EN');
                     $vCal->setAttribute('METHOD', 'REPLY');
                     $vCal->addComponent($vfb_reply);
 
-                    $mime = &new MIME_Message();
+                    $mime = new MIME_Message();
                     $message = _("Attached is a reply to a calendar request you sent.");
-                    $body = &new MIME_Part('text/plain',
-                                           String::wrap($message, 76, "\n"),
-                                           NLS::getCharset());
+                    $body = new MIME_Part('text/plain',
+                                          String::wrap($message, 76, "\n"),
+                                          NLS::getCharset());
 
-                    $ics = &new MIME_Part('text/calendar', $vCal->exportvCalendar());
+                    $ics = new MIME_Part('text/calendar', $vCal->exportvCalendar());
                     $ics->setName('icalendar.ics');
                     $ics->setContentTypeParameter('METHOD', 'REPLY');
+                    $ics->setCharset(NLS::getCharset());
 
                     $mime->addPart($body);
                     $mime->addPart($ics);
 
                     // Build the reply headers.
-                    $msg_headers = &new MIME_Headers();
+                    $msg_headers = new MIME_Headers();
                     $msg_headers->addReceivedHeader();
                     $msg_headers->addMessageIdHeader();
                     $msg_headers->addHeader('Date', date('r'));
@@ -448,7 +452,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
             }
         }
 
-        // Need to work out if we are inline and acutally need this.
+        // Need to work out if we are inline and actually need this.
         $html .= '</form>';
         if (MIME_Contents::viewAsAttachment()) {
             $html .= Util::bufferOutput('require', $registry->get('templates', 'horde') . '/common-footer.inc');
@@ -457,12 +461,14 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
         return $html;
     }
 
-    function _row($label, $value)
+    /**
+     * Return text/html as the content-type.
+     *
+     * @return string "text/html" constant
+     */
+    function getType()
     {
-        if (substr($label, 0, 2) == 'DT') {
-            $value = strftime("%x %X", $value);
-        }
-        return '<tr><td valign="top" class="item">' . $label . '</td><td valign="top" class="item">' . $value . "</td></tr>\n";
+        return 'text/html; charset=' . NLS::getCharset();
     }
 
     /**
@@ -472,53 +478,52 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
     {
         global $registry, $prefs;
 
-        $html = '<table cellspacing="1">';
-
+        $html = '';
         $desc = '';
-        $title = '';
         $sender = $vfb->getName();
         switch ($this->_method) {
         case 'PUBLISH':
             $desc = _("%s has sent you free/busy information.");
-            $title = _("Free/Busy Information");
             break;
 
         case 'REQUEST':
-            $desc = _("%s requests your free/busy information.");
-            $title = _("Free/Busy Request");
             $sender = $this->_headers->getValue('From');
+            $desc = _("%s requests your free/busy information.");
             break;
 
         case 'REPLY':
             $desc = _("%s has replied to a free/busy request.");
-            $title = _("Free/Busy Reply");
             break;
         }
 
-        $desc = sprintf($desc, $sender);
+        $html .= '<h1 class="header">' . sprintf($desc, $sender) . '</h1>';
 
-        $html .= '<tr><td colspan="2" class="header">' . $title . '</td></tr>' .
-            '<tr><td colspan="2" class="control">' . $desc . '<br />' .
-            _("Please select an action from the menu below.") . '</td></tr>';
-
-        if (count($this->_msgs)) {
+        if ($this->_msgs) {
             foreach ($this->_msgs[$id] as $msg) {
-                $html .= '<tr><td colspan="2" class="notice"><strong>' . Horde::img('alerts/' . $msg[0] . '.png', '', null, $registry->getImageDir('horde')) . $msg[1] . '</strong></td></tr>';
+                $html .= '<p class="notice">' . Horde::img('alerts/' . $msg[0] . '.png', '', null, $registry->getImageDir('horde')) . $msg[1] . '</p>';
             }
         }
 
         $start = $vfb->getAttribute('DTSTART');
         if (!is_a($start, 'PEAR_Error')) {
-            $html .= sprintf('<tr><td colspan="2" class="item"><b>%s:</b>&nbsp;%s</td></tr>', _("Start"), strftime($prefs->getValue('date_format') . ' ' . $prefs->getValue('time_format'), $start));
+            if (is_array($start)) {
+                $html .= '<p><strong>' . _("Start") . ':</strong> ' . strftime($prefs->getValue('date_format'), mktime(0, 0, 0, $start['month'], $start['mday'], $start['year'])) . '</p>';
+            } else {
+                $html .= '<p><strong>' . _("Start") . ':</strong> ' . strftime($prefs->getValue('date_format') . ' ' . $prefs->getValue('time_format'), $start) . '</p>';
+            }
         }
 
         $end = $vfb->getAttribute('DTEND');
         if (!is_a($end, 'PEAR_Error')) {
-            $html .= sprintf('<tr><td colspan="2" class="item"><b>%s:</b>&nbsp;%s</td></tr>', _("End"), strftime($prefs->getValue('date_format') . ' ' . $prefs->getValue('time_format'), $end));
+            if (is_array($end)) {
+                $html .= '<p><strong>' . _("End") . ':</strong> ' . strftime($prefs->getValue('date_format'), mktime(0, 0, 0, $end['month'], $end['mday'], $end['year'])) . '</p>';
+            } else {
+                $html .= '<p><strong>' . _("End") . ':</strong> ' . strftime($prefs->getValue('date_format') . ' ' . $prefs->getValue('time_format'), $end) . '</p>';
+            }
         }
 
-        $html .= '<tr><td colspan="2" class="control"><b>' . _("Actions") . ':</b><br />' .
-            _("Choose an action:") . "&nbsp;<select name='action[$id]'>";
+        $html .= '<h2 class="smallheader">' . _("Actions") . '</h2>' .
+            '<select name="action[' . $id . ']">';
 
         switch ($this->_method) {
         case 'PUBLISH':
@@ -549,10 +554,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
             break;
         }
 
-        $html .= sprintf('</select>&nbsp;<input type="submit" class="button" value="%s" /><br />', _("OK")) .
-            '</td></tr>' .
-            '</table>';
-        return $html;
+        return $html . '</select> <input type="submit" class="button" value="' . _("Go") . '/>';
     }
 
     /**
@@ -562,30 +564,25 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
     {
         global $registry, $prefs;
 
-        $html = '<table cellspacing="1">';
-
+        $html = '';
         $desc = '';
-        $title = '';
         $sender = $vevent->organizerName();
         $options = array();
 
         switch ($this->_method) {
         case 'PUBLISH':
-            $desc = _("%s wishes to make you aware of %s.");
-            $title = _("Meeting Information");
+            $desc = _("%s wishes to make you aware of \"%s\".");
             if ($registry->hasMethod('calendar/import')) {
                 $options[] = '<option value="import">' .   _("Add this to my calendar") . '</option>';
             }
-
             break;
 
         case 'REQUEST':
             // Check that you are one of the attendees here.
-            $desc = _("%s requests your presence at %s.");
-            $title = _("Meeting Proposal");
+            $desc = _("%s requests your presence at \"%s\".");
             if ($registry->hasMethod('calendar/import')) {
-                $options[] = '<option value="accept-import">' .   _("Accept request and add it to my calendar") . '</option>';
-                $options[] = '<option value="import">' .   _("Add this to my calendar") . '</option>';
+                $options[] = '<option value="accept-import">' .   _("Accept and add to my calendar") . '</option>';
+                $options[] = '<option value="import">' .   _("Add to my calendar") . '</option>';
             }
             $options[] = '<option value="accept">' . _("Accept request") . '</option>';
             $options[] = '<option value="tentative">' . _("Tentatively Accept request") . '</option>';
@@ -594,13 +591,14 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
             break;
 
         case 'ADD':
-            $desc = _("%s wishes to ammend %s.");
-            $title = _("Meeting Update");
+            $desc = _("%s wishes to ammend \"%s\".");
+            if ($registry->hasMethod('calendar/import')) {
+                $options[] = '<option value="import">' .   _("Update this event on my calendar") . '</option>';
+            }
             break;
 
         case 'REFRESH':
-            $desc = _("%s wishes to receive the latest information about %s.");
-            $title = _("Meeting Update Request");
+            $desc = _("%s wishes to receive the latest information about \"%s\".");
             $options[] = '<option value="send">' . _("Send Latest Information") . '</option>';
             if (!$found && $registry->hasMethod('calendar/eventFromGUID')) {
                 $existing_vevent = $registry->call('calendar/eventFromGUID', array('guid' => $vevent->getAttribute('UID')));
@@ -613,8 +611,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
             break;
 
         case 'REPLY':
-            $desc = _("%s has replied to the invitation to %s.");
-            $title = _("Meeting Reply");
+            $desc = _("%s has replied to the invitation to \"%s\".");
             $sender = $this->_headers->getValue('From');
             if ($registry->hasMethod('calendar/updateAttendee')) {
                 $options[] = '<option value="update">' . _("Update respondent status") . '</option>';
@@ -622,8 +619,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
             break;
 
         case 'CANCEL':
-            $desc = _("%s has cancelled %s.");
-            $title = _("Meeting Cancellation");
+            $desc = _("%s has cancelled \"%s\".");
             if ($registry->hasMethod('calendar/delete')) {
                 $options[] = '<option value="delete">' . _("Delete from my calendar") . '</option>';
             }
@@ -632,63 +628,67 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
 
         $summary = $vevent->getAttribute('SUMMARY');
         if (is_a($summary, 'PEAR_Error')) {
-            $desc = sprintf($desc, $sender, _("Unknown Meeting"));
+            $desc = sprintf($desc, htmlspecialchars($sender), _("Unknown Meeting"));
         } else {
-            $desc = sprintf($desc, $sender, $summary);
+            $desc = sprintf($desc, htmlspecialchars($sender), htmlspecialchars($summary));
         }
 
-        $html .= '<tr><td colspan="2" class="header">' . $title . '</td></tr>' .
-            '<tr><td colspan="2" class="control">' . $desc . '<br />' .
-            _("Please review the following information, and then select an action from the menu below.") . '</td></tr>';
+        $html .= '<h2 class="header">' . $desc . '</h2>';
 
-        if (count($this->_msgs)) {
+        if ($this->_msgs) {
             foreach ($this->_msgs[$id] as $msg) {
-                $html .= '<tr><td colspan="2" class="notice"><strong>' . Horde::img('alerts/' . $msg[0] . '.png', '', null, $registry->getImageDir('horde')) . $msg[1] . '</strong></td></tr>';
+                $html .= '<p class="notice">' . Horde::img('alerts/' . $msg[0] . '.png', '', null, $registry->getImageDir('horde')) . $msg[1] . '</p>';
             }
         }
 
         $start = $vevent->getAttribute('DTSTART');
         if (!is_a($start, 'PEAR_Error')) {
-            $html .= sprintf('<tr><td colspan="2" class="item"><b>%s:</b>&nbsp;%s</td></tr>', _("Start"), strftime($prefs->getValue('date_format') . ' ' . $prefs->getValue('time_format'), $start));
+            if (is_array($start)) {
+                $html .= '<p><strong>' . _("Start") . ':</strong> ' . strftime($prefs->getValue('date_format'), mktime(0, 0, 0, $start['month'], $start['mday'], $start['year'])) . '</p>';
+            } else {
+                $html .= '<p><strong>' . _("Start") . ':</strong> ' . strftime($prefs->getValue('date_format') . ' ' . $prefs->getValue('time_format'), $start) . '</p>';
+            }
         }
 
         $end = $vevent->getAttribute('DTEND');
         if (!is_a($end, 'PEAR_Error')) {
-            $html .= sprintf('<tr><td colspan="2" class="item"><b>%s:</b>&nbsp;%s</td></tr>', _("End"), strftime("%x %X", $end));
+            if (is_array($end)) {
+                $html .= '<p><strong>' . _("End") . ':</strong> ' . strftime($prefs->getValue('date_format'), mktime(0, 0, 0, $end['month'], $end['mday'], $end['year'])) . '</p>';
+            } else {
+                $html .= '<p><strong>' . _("End") . ':</strong> ' . strftime("%x %X", $end) . '</p>';
+            }
         }
 
         $sum = $vevent->getAttribute('SUMMARY');
         if (!is_a($sum, 'PEAR_Error')) {
-            $html .= sprintf('<tr><td colspan="2" class="item"><b>%s:</b>&nbsp;%s</td></tr>', _("Summary"), $sum);
+            $html .= '<p><strong>' . _("Summary") . ':</strong> ' . htmlspecialchars($sum) . '</p>';
         } else {
-            $html .= sprintf('<tr><td colspan="2" class="item"><b>%s:</b>&nbsp;<i>%s</i></td></tr>', _("Summary"), _("None"));
+            $html .= '<p><strong>' . _("Summary") . ':</strong> <em>' . _("None") . '</em></p>';
         }
 
         $desc = $vevent->getAttribute('DESCRIPTION');
         if (!is_a($desc, 'PEAR_Error')) {
-            $html .= sprintf('<tr><td colspan="2" class="item"><b>%s:</b><br />%s</td></tr>', _("Description"), preg_replace('/\\n/', '<br />', $desc));
+            $html .= '<p><strong>' . _("Description") . ':</strong> ' . nl2br(htmlspecialchars($desc)) . '</p>';
         }
 
         $loc = $vevent->getAttribute('LOCATION');
         if (!is_a($loc, 'PEAR_Error')) {
-            $html .= sprintf('<tr><td colspan="2" class="item"><b>%s:</b>&nbsp;%s</td></tr>', _("Location"), $loc);
+            $html .= '<p><strong>' . _("Location") . ':</strong>' . htmlspecialchars($loc) . '</p>';
         }
 
         $attendees = $vevent->getAttribute('ATTENDEE');
-        $params    = $vevent->getAttribute('ATTENDEE', true);
+        $params = $vevent->getAttribute('ATTENDEE', true);
 
         if (!is_a($attendees, 'PEAR_Error') && !empty($attendees)) {
-            $html .= sprintf('<tr><td colspan="2" class="item"><b>%s:</b><br />',
-                             _("Attendees"));
+            $html .= '<h2 class="smallheader">' . _("Attendees") . '</h2>';
             if (!is_array($attendees)) {
                 $attendees = array($attendees);
             }
 
-            $html .= sprintf('<table width="100%%"><tr><th align="left">%s</th><th align="left">%s</th><th align="left">%s</th></tr>',
-                             _("Name"), _("Role"), _("Status"));
+            $html .= '<table><thead class="leftAlign"><tr><th>' . _("Name") . '</th><th>' . _("Role") . '</th><th>' . _("Status") . '</th></tr></thead><tbody>';
             foreach ($attendees as $key => $attendee) {
                 $attendee = parse_url($attendee);
-                $attendee = $attendee['path'];
+                $attendee = empty($attendee['path']) ? _("Unknown") : $attendee['path'];
 
                 if (isset($params[$key]['CN'])) {
                     $attendee = $params[$key]['CN'];
@@ -721,20 +721,18 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                     $status = $this->_partstatToString($params[$key]['PARTSTAT'], $status);
                 }
 
-                $html .= sprintf('<tr><td>%s</td><td>%s</td><td>%s</td></tr>', htmlspecialchars($attendee), htmlspecialchars($role), htmlspecialchars($status));
+                $html .= '<tr><td>' . htmlspecialchars($attendee) . '</td><td>' . htmlspecialchars($role) . '</td><td>' . htmlspecialchars($status) . '</td></tr>';
             }
-            $html .= '</table></td></tr>';
+            $html .= '</tbody></table>';
         }
 
-        if (count($options)) {
-            $html .= '<tr><td colspan="2" class="control"><b>' . _("Actions") . ':</b><br />' .
-                _("Choose an action:") . "&nbsp;<select name='action[$id]'>" .
+        if ($options) {
+            $html .= '<h2 class="smallheader">' . _("Actions") . '</h2>' .
+                '<select name="action[' . $id . ']">' .
                 implode("\n", $options) .
-                '</select>&nbsp;<input type="submit" class="button" value="' . _("OK") . '" /><br />' .
-                '</td></tr>';
+                '</select> <input type="submit" class="button" value="' . _("Go") . '" />';
         }
 
-        $html .= '</table>';
         return $html;
     }
 
@@ -779,32 +777,4 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
         }
     }
 
-    /**
-     * Return text/html as the content-type.
-     *
-     * @return string "text/html" constant
-     */
-    function getType()
-    {
-        return 'text/html; charset=' . NLS::getCharset();
-    }
-
-    /**
-     * Get the headers of the message.
-     *
-     * @access private
-     *
-     * @param MIME_Contents &$contents  A MIME_Contents/IMP_Contents object.
-     */
-    function _getHeaders(&$contents)
-    {
-        if (empty($this->_headers)) {
-            if (!($this->_headers = $this->mime_part->getInformation('smime_headers'))) {
-                require_once IMP_BASE . '/lib/MIME/Headers.php';
-                $this->_headers = &new IMP_Headers($contents->getMessageIndex());
-                $this->_headers->buildHeaders();
-            }
-        }
-    }
-
 }
