@@ -10,11 +10,16 @@ define('IMP_MESSAGE_COPY', 2);
  * handled in here so that code need not be repeated between mailbox, message,
  * and other pages.
  *
- * $Horde: imp/lib/Message.php,v 1.164.8.8 2005/08/13 17:15:55 slusarz Exp $
+ * Indices format:
+ * ===============
+ * For any function below that requires an $indices parameter, see
+ * IMP::parseIndicesList() for the list of allowable inputs.
  *
- * Copyright 2000-2001 Chris Hyde <chris@jeks.net>
- * Copyright 2000-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * $Horde: imp/lib/Message.php,v 1.164.8.47 2007/01/02 13:54:56 jan Exp $
+ *
+ * Copyright 2000-2007 Chris Hyde <chris@jeks.net>
+ * Copyright 2000-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -41,14 +46,34 @@ class IMP_Message {
      */
     var $_usepop = false;
 
+    /**
+     * Returns a reference to the global IMP_Message object, only creating it
+     * if it doesn't already exist. This ensures that only one IMP_Message
+     * instance is instantiated for any given session.
+     *
+     * This method must be invoked as:<code>
+     *   $imp_message = &IMP_Message::singleton();
+     * </code>
+     *
+     * @return IMP_Message  The IMP_Message instance.
+     */
+    function &singleton()
+    {
+        static $message;
+
+        if (!isset($message)) {
+            $message = new IMP_Message();
+        }
+
+        return $message;
+    }
+
     /**
      * Constructor.
      */
     function IMP_Message()
     {
-        global $imp;
-
-        if ($imp['base_protocol'] == 'pop3') {
+        if ($GLOBALS['imp']['base_protocol'] == 'pop3') {
             $this->_usepop = true;
         }
 
@@ -56,13 +81,6 @@ class IMP_Message {
         $this->_impImap = &IMP_IMAP::singleton();
     }
 
-    /**
-     * Indices format:
-     * ===============
-     * For any function below that requires an $indices parameter, see
-     * IMP::parseIndicesList() for the list of allowable inputs.
-     */
-
     /**
      * Copies or moves a list of messages to a new folder.
      * Handles use of the IMP_SEARCH_MBOX mailbox and the Trash folder.
@@ -147,9 +165,16 @@ class IMP_Message {
             return false;
         }
 
+        $trash = IMP::folderPref($prefs->getValue('trash_folder'), true);
+        $use_trash = $prefs->getValue('use_trash');
+        if ($use_trash && !$trash) {
+            $notification->push(_("Can not move messages to Trash - no Trash mailbox set in preferences."), 'horde.error');
+            return false;
+        }
+
         $return_value = true;
-        $trash = IMP::addPreambleString($prefs->getValue('trash_folder'));
-        $use_trash = (!$nuke && $prefs->getValue('use_trash'));
+        $use_vtrash = $prefs->getValue('use_vtrash');
+        $maillog_update = (!$keeplog && !empty($conf['maillog']['use_maillog']));
 
         /* If the folder we are deleting from has changed. */
         foreach ($msgList as $folder => $msgIndices) {
@@ -159,9 +184,15 @@ class IMP_Message {
             $this->_impImap->changeMbox($folder);
 
             /* Trash is only valid for IMAP mailboxes. */
-            if (!$this->_usepop && $use_trash && ($folder != $trash)) {
-                include_once IMP_BASE . '/lib/Folder.php';
-                $imp_folder = &IMP_Folder::singleton();
+            if (!$this->_usepop &&
+                !$nuke &&
+                !$use_vtrash &&
+                $use_trash &&
+                ($folder != $trash)) {
+                if (!isset($imp_folder)) {
+                    include_once IMP_BASE . '/lib/Folder.php';
+                    $imp_folder = &IMP_Folder::singleton();
+                }
 
                 if (!$imp_folder->exists($trash)) {
                     if (!$imp_folder->create($trash, $prefs->getValue('subscribe'))) {
@@ -176,7 +207,8 @@ class IMP_Message {
                     /* Handle the case when the mailbox is overquota
                        (moving message to trash would fail) by first
                        deleting then copying message to Trash. */
-                    if (stristr('over quota', $error_msg) !== false) {
+                    if ((stristr($error_msg, 'over quota') !== false) ||
+                        (stristr($error_msg, 'exceeded your mail quota') !== false)) {
                         $error = false;
                         $msg_text = array();
 
@@ -189,7 +221,7 @@ class IMP_Message {
 
                         /* Save messages in Trash folder. */
                         foreach ($msg_text as $val) {
-                            if (!@imap_append($imp['stream'], $trash, $val)) {
+                            if (!@imap_append($imp['stream'], IMP::serverString($trash), $val)) {
                                 $error = true;
                                 break;
                             }
@@ -204,8 +236,11 @@ class IMP_Message {
                     @imap_expunge($imp['stream']);
                 }
             } else {
-                /* Get the list of Message-IDs for the deleted messages. */
-                $overview = @imap_fetch_overview($imp['stream'], $sequence, FT_UID);
+                /* Get the list of Message-IDs for the deleted messages if
+                 * using maillogging. */
+                if ($maillog_update) {
+                    $overview = @imap_fetch_overview($imp['stream'], $sequence, FT_UID);
+                }
 
                 /* Delete the messages. */
                 if (!@imap_delete($imp['stream'], $sequence, FT_UID)) {
@@ -216,16 +251,28 @@ class IMP_Message {
                     }
                     $return_value = false;
                 } else {
-                    if ($nuke || ($use_trash && ($folder == $trash))) {
+                    $delIndices = array($folder => $msgIndices);
+                    $flag_str = '\\DELETED';
+
+                    if ($this->_usepop ||
+                        $nuke ||
+                        ($use_trash && ($folder == $trash))) {
                         /* Purge messages in the trash folder immediately. */
+                        $this->flag($flag_str, $delIndices);
                         @imap_expunge($imp['stream']);
-                    } elseif ($this->_usepop) {
-                        $this->_impImap->reopenIMAPStream(true);
+                    } else {
+                        /* If we are using vitrual trash, we must mark the 
+                         * message as seen or else it will appear as an
+                         * 'unseen' message for purposes of new message
+                         * counts. */
+                        if ($use_vtrash) {
+                            $this->flag($flag_str . ' \\SEEN', $delIndices);
+                        }
                     }
 
                     /* Get the list of Message-IDs deleted, and remove
                      * the information from the mail log. */
-                    if (!$keeplog && !empty($conf['maillog']['use_maillog'])) {
+                    if ($maillog_update) {
                         $msg_ids = array();
                         foreach ($overview as $val) {
                             if (!empty($val->message_id)) {
@@ -288,17 +335,18 @@ class IMP_Message {
     }
 
     /**
-     * Copies or moves a list of messages to a tasklist.
+     * Copies or moves a list of messages to a tasklist or notepad.
      * Handles use of the IMP_SEARCH_MBOX mailbox and the Trash folder.
      *
-     * @param string    $tasklist   The tasklist in which the task will be
-     *                              created.
-     * @param integer   $action     Either IMP_MESSAGE_MOVE or IMP_MESSAGE_COPY.
-     * @param mixed     &$indices   See above.
+     * @param string $list      The list in which the task or note will be
+     *                          created.
+     * @param integer $action   Either IMP_MESSAGE_MOVE or IMP_MESSAGE_COPY.
+     * @param mixed $indices    See above.
+     * @param string $type      The object type to create, defaults to task.
      *
      * @return boolean  True if successful, false if not.
      */
-    function createTasks($tasklist, $action, &$indices)
+    function createTasksOrNotes($list, $action, &$indices, $type = 'task')
     {
         global $registry, $notification, $prefs;
 
@@ -320,7 +368,7 @@ class IMP_Message {
                 $subject = $imp_headers->getValue('subject');
 
                 /* Fetch the message contents. */
-                $imp_contents = &IMP_Contents::singleton($index);
+                $imp_contents = &IMP_Contents::singleton($index . IMP_IDX_SEP . $folder);
                 $imp_contents->buildMessage();
 
                 /* Extract the message body. */
@@ -331,6 +379,10 @@ class IMP_Message {
 
                 /* Re-flow the message for prettier formatting. */
                 $flowed = &new Text_Flowed($mime_message->replaceEOL($body, "\n"));
+                if (($mime_message->getContentTypeParameter('delsp') == 'yes') &&
+                    method_exists($flowed, 'setDelSp')) {
+                    $flowed->setDelSp(true);
+                }
                 $body = $flowed->toFlowed(false);
 
                 /* Convert to current charset */
@@ -345,36 +397,74 @@ class IMP_Message {
                 $vCal->setAttribute('PRODID', '-//The Horde Project//IMP ' . IMP_VERSION . '//EN');
                 $vCal->setAttribute('METHOD', 'PUBLISH');
 
-                /* Create a new vTodo object using this message's contents. */
-                $vTodo = &Horde_iCalendar::newComponent('vtodo', $vCal);
-                $vTodo->setAttribute('SUMMARY', $subject);
-                $vTodo->setAttribute('DESCRIPTION', $body);
-                $vTodo->setAttribute('PRIORITY', '3');
+                switch ($type) {
+                case 'task':
+                    /* Create a new vTodo object using this message's
+                     * contents. */
+                    $vTodo = &Horde_iCalendar::newComponent('vtodo', $vCal);
+                    $vTodo->setAttribute('SUMMARY', $subject);
+                    $vTodo->setAttribute('DESCRIPTION', $body);
+                    $vTodo->setAttribute('PRIORITY', '3');
 
-                /* Get the list of editable tasklists. */
-                $tasklists = $registry->call('tasks/listTasklists',
+                    /* Get the list of editable tasklists. */
+                    $lists = $registry->call('tasks/listTasklists',
                                              array(false, PERMS_EDIT));
 
-                /* Attempt to add the new vTodo item to the requested tasklist. */
-                $res = $registry->call('tasks/import',
-                                       array($vTodo, 'text/x-vtodo', $tasklist));
+                    /* Attempt to add the new vTodo item to the requested
+                     * tasklist. */
+                    $res = $registry->call('tasks/import',
+                                           array($vTodo, 'text/x-vtodo', $list));
+                    break;
+
+                case 'note':
+                    /* Create a new vNote object using this message's
+                     * contents. */
+                    $vNote = &Horde_iCalendar::newComponent('vnote', $vCal);
+                    $vNote->setAttribute('BODY', $subject . "\n". $body);
+
+                    /* Get the list of editable notepads. */
+                    $lists = $registry->call('notes/listNotepads',
+                                             array(false, PERMS_EDIT));
+
+                    /* Attempt to add the new vNote item to the requested
+                     * notepad. */
+                    $res = $registry->call('notes/import',
+                                           array($vNote, 'text/x-vnote', $list));
+                    break;
+                }
+
                 if (is_a($res, 'PEAR_Error')) {
                     $notification->push($res, $res->getCode());
                 } elseif (!$res) {
-                    $notification->push(_("An unknown error occured while creating the new task."), 'horde.error');
+                    switch ($type) {
+                    case 'task': $notification->push(_("An unknown error occured while creating the new task."), 'horde.error'); break;
+                    case 'note': $notification->push(_("An unknown error occured while creating the new note."), 'horde.error'); break;
+                    }
                 } else {
-                    $task_name = '"' . htmlspecialchars($subject) . '"';
+                    $name = '"' . htmlspecialchars($subject) . '"';
 
-                    /* Attempt to convert the task name into a hyperlink. */
-                    $task_link = $registry->link('tasks/show',
-                                                 array('uid' => $res));
-                    if ($task_link && !is_a($task_link, 'PEAR_Error')) {
-                        $task_name = sprintf('<a href="%s">%s</a>',
-                                             Horde::url($task_link),
-                                             $task_name);
+                    /* Attempt to convert the object name into a hyperlink. */
+                    switch ($type) {
+                    case 'task':
+                        $link = $registry->link('tasks/show',
+                                                array('uid' => $res));
+                        break;
+                    case 'note':
+                        if ($registry->hasMethod('notes/show')) {
+                            $link = $registry->link('notes/show',
+                                                    array('uid' => $res));
+                        } else {
+                            $link = false;
+                        }
+                        break;
+                    }
+                    if ($link && !is_a($link, 'PEAR_Error')) {
+                        $name = sprintf('<a href="%s">%s</a>',
+                                        Horde::url($link),
+                                        $name);
                     }
 
-                    $notification->push(sprintf(_("%s was successfully added to %s."), $task_name, htmlspecialchars($tasklists[$tasklist]->get('name'))), 'horde.success', array('content.raw'));
+                    $notification->push(sprintf(_("%s was successfully added to \"%s\"."), $name, htmlspecialchars($lists[$list]->get('name'))), 'horde.success', array('content.raw'));
                 }
             }
         }
@@ -421,12 +511,15 @@ class IMP_Message {
 
         /* Get a local copy of the message and strip out the desired
            MIME_Part object. */
-        $contents = &IMP_Contents::singleton($index);
+        $contents = &IMP_Contents::singleton($index . IMP_IDX_SEP . $folder);
         $contents->rebuildMessage();
         $message = $contents->getMIMEMessage();
         $oldPart = $message->getPart($partid);
         $newPart = new MIME_Part('text/plain');
-        $newPart->setContents('[' . _("Attachment stripped: Original attachment type") . ': "' . $oldPart->getType() . '", ' . _("name") . ': "' . $oldPart->getName(true, true) . '"]', '8bit');
+
+        /* We need to make sure all text is in the correct charset. */
+        $newPart->setCharset(NLS::getCharset());
+        $newPart->setContents(sprintf(_("[Attachment stripped: Original attachment type: %s, name: %s]"), $oldPart->getType(), $oldPart->getName(true, true)), '8bit');
         $message->alterPart($partid, $newPart);
 
         /* We need to make sure we add "\r\n" after every line for
@@ -435,12 +528,11 @@ class IMP_Message {
 
         /* Get the headers for the message. */
         $headers = &new IMP_Headers($index);
-        $headertext = $headers->getHeaderText();
         $headers->buildFlags();
         $flags = array();
-        foreach (array('answered', 'draft', 'flagged', 'deleted') as $flag) {
+        foreach (array('answered', 'deleted', 'draft', 'flagged', 'seen') as $flag) {
             if ($headers->getFlag($flag)) {
-                $flags[] = '\\' . String::ucfirst($flag);
+                $flags[] = '\\' . $flag;
             }
         }
         $flags = implode(' ', $flags);
@@ -449,19 +541,17 @@ class IMP_Message {
            we check imap_status() to determine the next available UID. We
            use this UID as the new index of the message. */
         $folderstring = IMP::serverString($folder);
-        $this->_impImap->changeMbox($folderstring);
+        $headerstring = $headers->getHeaderText();
+        $this->_impImap->changeMbox($folder);
         $status = @imap_status($imp['stream'], $folderstring, SA_UIDNEXT);
-        if (@imap_append($imp['stream'], $folderstring, $headertext . $message->toString(), '\\SEEN')) {
-            $idx_array = array($folder => $index);
-            $this->delete($idx_array, true, true);
-            $idx_array = array($folder => $status->uidnext);
-            $this->flag($flags, $idx_array);
-            $imp_mailbox->setNewIndex($status->uidnext);
+        if (@imap_append($imp['stream'], $folderstring, $headerstring . $contents->toString($message, true), $flags)) {
+            $this->delete($imp_mailbox, true, true);
             $imp_mailbox->updateMailbox(IMP_MAILBOX_UPDATE);
+            $imp_mailbox->setNewIndex($status->uidnext);
 
             /* We need to replace the old index in the query string with the
                new index. */
-            $_SERVER['QUERY_STRING'] = preg_replace('/' . $index . '/', $imp_mailbox->getIndex(), $_SERVER['QUERY_STRING']);
+            $_SERVER['QUERY_STRING'] = preg_replace('/' . $index . '/', $status->uidnext, $_SERVER['QUERY_STRING']);
 
             return true;
         } else {
@@ -517,12 +607,7 @@ class IMP_Message {
      * Sets or clears a given flag(s) for all messages in a list of mailboxes.
      * This function works with IMAP only, not POP3.
      *
-     * Valid flags are:
-     *   \\SEEN
-     *   \\FLAGGED
-     *   \\ANSWERED
-     *   \\DELETED
-     *   \\DRAFT
+     * See flag() for the list of valid flags.
      *
      * @param string $flag     The IMAP flag(s) to set or clear.
      * @param array $mboxes    The list of mailboxes to flag.
@@ -537,16 +622,16 @@ class IMP_Message {
             return false;
         }
 
-        $function = ($action) ? 'imap_setflag_full' : 'imap_clearflag_full';
         $return_value = true;
 
         foreach ($mboxes as $val) {
             /* Switch folders, if necessary. */
             $this->_impImap->changeMbox($val);
-
-            if (!call_user_func($function, $GLOBALS['imp']['stream'], '1:*', $flag)) {
-                $notification->push(sprintf(_("There was an error flagging messages in the folder \"%s\". This is what the server said"), IMP::displayFolder($val)) . ': ' . imap_last_error(), 'horde.error');
-                $return_value = false;
+            if ($uids = @imap_search($GLOBALS['imp']['stream'], 'ALL', SE_UID)) {
+                $indices = array($val => $uids);
+                if (!$this->flag($flag, $indices, $action)) {
+                    $return_value = false;
+                }
             }
         }
 
@@ -560,29 +645,18 @@ class IMP_Message {
      */
     function expungeMailbox($mbox_list)
     {
-        global $imp, $notification;
+        global $imp;
 
         foreach ($mbox_list as $val) {
-            if ($val == IMP_SEARCH_MBOX) {
-                for ($i = 0; $i < count($imp['search']['folders']); $i++) {
-                    if ($this->_usepop) {
-                        $stream = $imp['stream'];
-                    } else {
-                        $stream = $this->_impImap->openIMAPStream($imp['search']['folders'][$i]);
-                    }
-
-                    if (!@imap_expunge($stream)) {
-                        $notification->push(sprintf(_("There was a problem expunging %s. This is what the server said"), IMP::displayFolder($imp['search']['folders'][$i])) . ': ' . imap_last_error(), 'horde.error');
-                    }
-                    if (!$this->_usepop) {
-                        @imap_close($stream);
-                    }
+            if ($GLOBALS['imp_search']->isSearchMbox($val)) {
+                foreach ($GLOBALS['imp_search']->getSearchFolders($val) as $folder) {
+                    $stream = $this->_impImap->openIMAPStream($folder);
+                    @imap_expunge($stream);
+                    @imap_close($stream);
                 }
             } else {
                 $this->_impImap->changeMbox($val);
-                if (!@imap_expunge($imp['stream'])) {
-                    $notification->push(_("There was a problem expunging the mailbox. This is what the server said") . ': ' . imap_last_error(), 'horde.error');
-                }
+                @imap_expunge($imp['stream']);
             }
         }
     }
@@ -597,6 +671,12 @@ class IMP_Message {
         global $imp, $notification;
 
         foreach ($mbox_list as $mbox) {
+            if ($GLOBALS['imp_search']->isVTrashFolder($mbox)) {
+                $this->expungeMailbox($GLOBALS['imp_search']->getSearchFolders($mbox));
+                $notification->push(_("Emptied all messages from Virtual Trash Folder."), 'horde.success');
+                continue;
+            }
+
             $display_mbox = IMP::displayFolder($mbox);
 
             if (($this->_impImap->changeMbox($mbox)) !== true) {
@@ -606,11 +686,8 @@ class IMP_Message {
 
             /* Make sure there is at least 1 message before attempting to
                delete. */
-            $check = @imap_check($imp['stream']);
-            $delete_array = array($mbox => '1:*');
-            if (!is_object($check)) {
-                $notification->push(sprintf(_("%s does not appear to be a valid mailbox."), $display_mbox), 'horde.error');
-            } elseif (empty($check->Nmsgs)) {
+            $delete_array = array($mbox => array('1:*'));
+            if (!@imap_num_msg($imp['stream'])) {
                 $notification->push(sprintf(_("The mailbox %s is already empty."), $display_mbox), 'horde.message');
             } elseif (!$this->delete($delete_array, true)) {
                 $notification->push(sprintf(_("There was a problem expunging the mailbox. The server said: %s"), imap_last_error()), 'horde.error');
