@@ -1,9 +1,9 @@
 <?php
 /**
- * $Horde: imp/mailbox.php,v 2.617.4.21 2005/08/15 22:01:19 jan Exp $
+ * $Horde: imp/mailbox.php,v 2.617.4.66 2007/01/02 13:54:54 jan Exp $
  *
- * Copyright 1999-2005 Charles J. Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2007 Charles J. Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (GPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -18,11 +18,9 @@ function _deleteMessages($indices)
 
     if (!empty($indices)) {
         require_once IMP_BASE . '/lib/Message.php';
-        $imp_message = &new IMP_Message();
+        $imp_message = &IMP_Message::singleton();
         $imp_message->delete($indices);
 
-        /* For POP3, redirect so that refreshing the page doesn't delete more
-           messages accidentally. */
         if ($imp['base_protocol'] == 'pop3') {
             header('Location: ' . Horde::url($mailbox_url, true));
             exit;
@@ -59,7 +57,7 @@ function _outputSummaries($search_mbox, $msgs)
 {
     $template = &new Horde_Template();
     $template->set('idx_separator', IMP_IDX_SEP);
-    $template->set('search_mbox', $search_mbox, true);
+    $template->set('search_mbox', (bool) $search_mbox, true);
     $template->set('messages', $msgs, true);
 
     // Some browsers have trouble with hidden overflow in table cells
@@ -74,35 +72,6 @@ function _outputSummaries($search_mbox, $msgs)
     echo $template->fetch(IMP_TEMPLATES . '/mailbox/mailbox.html');
 }
 
-/**
- * Utility function to return a url for the various images.
- *
- * @param string $name  Image filename.
- *
- * @return string  Link to the folder image.
- */
-function _image($name)
-{
-    global $graphicsdir;
-    static $cache = array();
-
-    if (!empty($cache[$name])) {
-        return $cache[$name];
-    }
-
-    $cache[$name] = Horde::img('tree/' . $name, '', 'style="vertical-align:middle"', $graphicsdir);
-
-    return $cache[$name];
-}
-
-$do_filter = false;
-$open_compose_window = false;
-$set = false;
-if (isset($new_lang)) {
-    $language = $new_lang;
-    $set = true;
-}
-
 @define('IMP_BASE', dirname(__FILE__));
 require_once IMP_BASE . '/lib/base.php';
 require_once IMP_BASE . '/lib/Mailbox.php';
@@ -119,7 +88,7 @@ if (!empty($conf['hooks']['mbox_redirect'])) {
         $redirect = call_user_func('_imp_hook_mbox_redirect', $imp['mailbox']);
         if (!empty($redirect)) {
             $redirect = Horde::applicationUrl($redirect, true);
-            header("Location: " . $redirect);
+            header('Location: ' . $redirect);
             exit;
         }
     }
@@ -128,12 +97,12 @@ if (!empty($conf['hooks']['mbox_redirect'])) {
 /* There is a chance that this page is loaded directly via
  * message.php. If so, don't re-include config files, and the
  * following variables will already be set: $actionID, $start. */
-if (!isset($from_message_page)) {
+if (isset($from_message_page)) {
+    $mailbox_url = Horde::applicationURL('mailbox.php');
+} else {
+    $mailbox_url = Horde::selfUrl();
     $actionID = Util::getFormData('actionID');
-}
-
-if ($actionID == 'login_compose') {
-    $open_compose_window = IMP::openComposeWin();
+    $start = Util::getFormData('start');
 }
 
 /* Get form data and make sure it's the type that we're expecting. */
@@ -153,31 +122,30 @@ $charset = NLS::getCharset();
 /* Initialize the user's identities. */
 $identity = &Identity::singleton(array('imp', 'imp'));
 
-/* Get the base URL for this page. */
-if (isset($from_message_page)) {
-    $mailbox_url = Horde::applicationURL('mailbox.php');
-} else {
-    $mailbox_url = Horde::selfUrl();
-}
-
-/* Check for 'sortby' and 'sortdir' changes. */
-if (($sortby = Util::getFormData('sortby')) !== null) {
-    $prefs->setValue('sortby', $sortby);
-}
-if (($sortdir = Util::getFormData('sortdir')) !== null) {
-    $prefs->setValue('sortdir', $sortdir);
-}
+$do_filter = $open_compose_window = false;
 
 /* Run through the action handlers */
 switch ($actionID) {
+case 'change_sort':
+    if (($sortby = Util::getFormData('sortby')) !== null) {
+        $prefs->setValue('sortby', $sortby);
+    }
+    if (($sortdir = Util::getFormData('sortdir')) !== null) {
+        $prefs->setValue('sortdir', $sortdir);
+    }
+    $imp_mailbox = &IMP_Mailbox::singleton(null, true);
+    break;
+
 case 'blacklist':
     require_once IMP_BASE . '/lib/Filter.php';
-    IMP_Filter::blacklistMessage($indices_mbox);
+    $imp_filter = &IMP_Filter::singleton();
+    $imp_filter->blacklistMessage($indices_mbox);
     break;
 
 case 'whitelist':
     require_once IMP_BASE . '/lib/Filter.php';
-    IMP_Filter::whitelistMessage($indices_mbox);
+    $imp_filter = &IMP_Filter::singleton();
+    $imp_filter->whitelistMessage($indices_mbox);
     break;
 
 case 'spam_report':
@@ -188,7 +156,8 @@ case 'notspam_report':
     $imp_spam->reportSpam($indices_mbox, $action);
 
     /* Delete spam after report. */
-    if (($action == 'spam') && $prefs->getValue('delete_spam_after_report')) {
+    $delete_spam = $prefs->getValue('delete_spam_after_report');
+    if (($action == 'spam') && ($delete_spam == 1)) {
         if (_deleteMessages($indices_mbox)) {
             if (count($indices) == 1) {
                 $notification->push(_("1 message has been deleted."), 'horde.message');
@@ -197,6 +166,11 @@ case 'notspam_report':
                                             count($indices)), 'horde.message');
             }
         }
+    } elseif ($delete_spam == 2) {
+        $targetMbox = ($action == 'spam') ? IMP::folderPref($prefs->getValue('spam_folder'), true) : 'INBOX';
+        require_once IMP_BASE . '/lib/Message.php';
+        $imp_message = &IMP_Message::singleton();
+        $imp_message->copy($targetMbox, IMP_MESSAGE_MOVE, $indices_mbox, true);
     }
     break;
 
@@ -205,8 +179,7 @@ case 'message_missing':
     break;
 
 case 'fwd_digest':
-    $options['fwddigest'] = serialize($indices);
-    $options['actionID'] = 'fwd_digest';
+    $options = array('fwddigest' => serialize($indices), 'actionID' => 'fwd_digest');
     $open_compose_window = IMP::openComposeWin($options);
     break;
 
@@ -217,7 +190,7 @@ case 'delete_messages':
 case 'undelete_messages':
     if (!empty($indices_mbox)) {
         require_once IMP_BASE . '/lib/Message.php';
-        $imp_message = &new IMP_Message();
+        $imp_message = &IMP_Message::singleton();
         $imp_message->undelete($indices_mbox);
     }
     break;
@@ -226,18 +199,23 @@ case 'move_messages':
 case 'copy_messages':
     if (!empty($indices) && !empty($targetMbox)) {
         require_once IMP_BASE . '/lib/Message.php';
-        $imp_message = &new IMP_Message();
+        $imp_message = &IMP_Message::singleton();
         $action = ($actionID == 'move_messages') ? IMP_MESSAGE_MOVE : IMP_MESSAGE_COPY;
 
         if ($conf['tasklist']['use_tasklist'] &&
             (strpos($targetMbox, '_tasklist_') === 0)) {
             /* If the target is a tasklist, handle the move/copy specially. */
             $tasklist = str_replace('_tasklist_', '', $targetMbox);
-            $imp_message->createTasks($tasklist, $action, $indices_mbox);
+            $imp_message->createTasksOrNotes($tasklist, $action, $indices_mbox, 'task');
+        } elseif ($conf['notepad']['use_notepad'] &&
+            (strpos($targetMbox, '_notepad_') === 0)) {
+            /* If the target is a notepad, handle the move/copy specially. */
+            $notepad = str_replace('_notepad_', '', $targetMbox);
+            $imp_message->createTasksOrNotes($notepad, $action, $indices_mbox, 'note');
         } else {
             /* Otherwise, the target is a standard mailbox. */
             if (!empty($newMbox) && ($newMbox == 1)) {
-                $new_mailbox = IMP::addPreambleString(String::convertCharset($targetMbox, $charset, 'UTF7-IMAP'));
+                $new_mailbox = String::convertCharset(IMP::folderPref($targetMbox, true), $charset, 'UTF7-IMAP');
 
                 require_once IMP_BASE . '/lib/Folder.php';
                 $imp_folder = &IMP_Folder::singleton();
@@ -254,7 +232,7 @@ case 'copy_messages':
 case 'flag_messages':
     $flag = Util::getPost('flag');
     if (!empty($indices) && !empty($flag)) {
-        if ($flag[0] == '0') {
+        if ($flag{0} == '0') {
             $_POST['flag'] = '\\' . substr($flag, 1);
             $set = false;
         } else {
@@ -262,7 +240,7 @@ case 'flag_messages':
             $set = true;
         }
         require_once IMP_BASE . '/lib/Message.php';
-        $imp_message = &new IMP_Message();
+        $imp_message = &IMP_Message::singleton();
         $imp_message->flag($_POST['flag'], $indices_mbox, $set);
     }
     break;
@@ -273,7 +251,7 @@ case 'hide_deleted':
 
 case 'expunge_mailbox':
     require_once IMP_BASE . '/lib/Message.php';
-    $imp_message = &new IMP_Message();
+    $imp_message = &IMP_Message::singleton();
     $imp_message->expungeMailbox(array($imp['mailbox']));
     break;
 
@@ -283,18 +261,25 @@ case 'filter':
 
 case 'empty_mailbox':
     require_once IMP_BASE . '/lib/Message.php';
-    $mbox = Util::getFormData('thismailbox', $imp['mailbox']);
-    $imp_message = &new IMP_Message();
-    $imp_message->emptyMailbox(array($mbox));
-    if ($url = Util::getFormData('return_url')) {
-        header('Location: ' . str_replace('&amp;', '&', $url));
-        exit;
-    }
+    $imp_message = &IMP_Message::singleton();
+    $imp_message->emptyMailbox(array($imp['mailbox']));
+    break;
+
+case 'view_messages':
+    require_once 'Horde/SessionObjects.php';
+    $cacheSess = &Horde_SessionObjects::singleton();
+    $redirect = Util::addParameter(Horde::applicationUrl('thread.php', true), array('mode' => 'msgview', 'msglist' => $cacheSess->storeOid($indices_mbox)), null, false);
+    header("Location: " . $redirect);
+    exit;
+    break;
+
+case 'login_compose':
+    $open_compose_window = IMP::openComposeWin();
     break;
 }
 
 /* Is this a search mailbox? */
-$search_mbox = ($imp['mailbox'] == IMP_SEARCH_MBOX);
+$search_mbox = $imp_search->searchMboxID();
 
 /* Deal with filter options. */
 if ($imp['filteravail']) {
@@ -310,53 +295,58 @@ if ($imp['filteravail']) {
 /* Run filters now. */
 if ($do_filter) {
     require_once IMP_BASE . '/lib/Filter.php';
-    IMP_Filter::filter($imp['mailbox']);
+    $imp_filter = &IMP_Filter::singleton();
+    $imp_filter->filter($imp['mailbox']);
 }
 
 /* Generate folder options list. */
 if ($conf['user']['allow_folders']) {
-    $folder_options = IMP::flistSelect(_("Messages to"), true, array(null, $imp['mailbox']), null, true, true);
-}
-
-/* Get the start index. It may already be set from message.php. */
-if (!isset($from_message_page)) {
-    $start = Util::getFormData('start');
+    $folder_options = IMP::flistSelect(_("Messages to"), true, array(), null, true, true, false, true);
 }
 
 /* Build the list of messages in the mailbox. */
 if (!isset($imp_mailbox)) {
-    $imp_mailbox = &new IMP_Mailbox();
+    $imp_mailbox = &IMP_Mailbox::singleton();
 }
+$imp_mailbox->setNoNewmailPopup(Util::getFormData('no_newmail_popup'));
 $pageOb = $imp_mailbox->buildMailboxPage(Util::getFormData('page'), $start);
+$mailboxOverview = $imp_mailbox->buildMailboxArray($pageOb->begin, $pageOb->end);
 
 /* Cache this value since we use it alot on this page. */
 $graphicsdir = $registry->getImageDir('horde');
 
 /* Generate First/Previous page links. */
 if ($pageOb->page == 1) {
-    $pages_first = Horde::img('nav/first-grey.png', '', 'width="16" height="16" align="middle"', $graphicsdir);
-    $pages_prev = Horde::img('nav/left-grey.png', '', 'width="16" height="16" align="middle"', $graphicsdir);
+    $pages_first = Horde::img('nav/first-grey.png', null, null, $graphicsdir);
+    $pages_prev = Horde::img('nav/left-grey.png', null, null, $graphicsdir);
 } else {
     $first_url = Util::addParameter($mailbox_url, 'page', 1);
-    $pages_first = Horde::link($first_url, _("First Page")) . Horde::img('nav/first.png', '<<', array('title' => _("First Page"), 'width' => '16', 'height' => '16', 'align' => 'middle'), $graphicsdir) . '</a>';
+    $pages_first = Horde::link($first_url, _("First Page")) . Horde::img('nav/first.png', '<<', null, $graphicsdir) . '</a>';
     $prev_url = Util::addParameter($mailbox_url, 'page', $pageOb->page - 1);
-    $pages_prev = Horde::link($prev_url, _("Previous Page"), '', '', '', '', '', array('id' => 'prev')) . Horde::img('nav/left.png', '<', array('title' => _("Previous Page"), 'width' => '16', 'height' => '16', 'align' => 'middle'), $graphicsdir) . '</a>';
+    $pages_prev = Horde::link($prev_url, _("Previous Page"), '', '', '', '', '', array('id' => 'prev')) . Horde::img('nav/left.png', '<', null, $graphicsdir) . '</a>';
 }
 
 /* Generate Next/Last page links. */
 if ($pageOb->page == $pageOb->pagecount) {
-    $pages_last = Horde::img('nav/last-grey.png', '', 'width="16" height="16" align="middle"', $graphicsdir);
-    $pages_next = Horde::img('nav/right-grey.png', '', 'width="16" height="16" align="middle"', $graphicsdir);
+    $pages_last = Horde::img('nav/last-grey.png', null, null, $graphicsdir);
+    $pages_next = Horde::img('nav/right-grey.png', null, null, $graphicsdir);
 } else {
     $next_url = Util::addParameter($mailbox_url, 'page', $pageOb->page + 1);
-    $pages_next = Horde::link($next_url, _("Next Page"), '', '', '', '', '', array('id' => 'next')) . Horde::img('nav/right.png', '>', array('title' => _("Next Page"), 'width' => '16', 'height' => '16', 'align' => 'middle'), $graphicsdir) . '</a>';
+    $pages_next = Horde::link($next_url, _("Next Page"), '', '', '', '', '', array('id' => 'next')) . Horde::img('nav/right.png', '>', null, $graphicsdir) . '</a>';
     $last_url = Util::addParameter($mailbox_url, 'page', $pageOb->pagecount);
-    $pages_last = Horde::link($last_url, _("Last Page")) . Horde::img('nav/last.png', '>>', array('title' => _("Last Page"), 'width' => '16', 'height' => '16', 'align' => 'middle'), $graphicsdir) . '</a>';
+    $pages_last = Horde::link($last_url, _("Last Page")) . Horde::img('nav/last.png', '>>', null, $graphicsdir) . '</a>';
 }
 
 /* Determine if we are going to show the Hide/Purge Deleted Message links. */
-$showdelete = $imp_mailbox->showDeleteLinks();
-if ($showdelete && !$prefs->isLocked('delhide')) {
+$sortby = $imp_mailbox->sortby();
+if (!$prefs->getValue('use_trash') &&
+    !$prefs->getValue('use_vtrash') &&
+    !$GLOBALS['imp_search']->isVINBOXFolder()) {
+    $showdelete = array('hide' => ($sortby != SORTTHREAD), 'purge' => true);
+} else {
+    $showdelete = array('hide' => false, 'purge' => false);
+}
+if ($showdelete['hide'] && !$prefs->isLocked('delhide')) {
     if ($prefs->getValue('delhide')) {
         $deleted_prompt = _("Show Deleted");
     } else {
@@ -391,11 +381,24 @@ if ($preview_tooltip) {
     $strip_preview = $prefs->getValue('preview_strip_nl');
 }
 
-
+$vtrash = null;
 if ($search_mbox) {
     $unread = 0;
+    if ($imp_search->isVINBOXFolder()) {
+        $unread = $imp_mailbox->getMessageCount();
+    } elseif ($imp_search->isVTrashFolder()) {
+        $vtrash = $imp_search->createSearchID($search_mbox);
+   }
 } else {
-    $unread = $imp_mailbox->unseenMessageCount();
+    $unread = count($imp_mailbox->unseenMessages());
+}
+
+/* Play audio. */
+if ($prefs->getValue('nav_audio') &&
+    ($newmsgs = $imp_mailbox->newMessageCount()) &&
+    ($newmsgs > 0) &&
+    !Util::getFormData('no_newmail_popup')) {
+    $notification->push($registry->getImageDir() . '/audio/theetone.wav', 'audio');
 }
 
 $title = IMP::getLabel();
@@ -426,13 +429,11 @@ if (isset($imp['quota']) && is_array($imp['quota'])) {
     }
 }
 
-/* Build the array of message information. */
-$mailboxOverview = $imp_mailbox->buildMailboxArray($pageOb->begin, $pageOb->index, $pageOb->end, $imp_mailbox->getSorted(), $imp_mailbox->getSortedMbox());
-
 /* Are we currently in the trash mailbox? */
-$preamble = IMP::preambleString();
 $use_trash = $prefs->getValue('use_trash');
-$trashMbox = ($use_trash && ($imp['mailbox'] == ($preamble . $prefs->getValue('trash_folder'))));
+$use_vtrash = $prefs->getValue('use_vtrash');
+$trashMbox = ($use_trash && ($imp['mailbox'] == (IMP::folderPref($prefs->getValue('trash_folder'), true)))) ||
+             ($use_vtrash && !is_null($vtrash));
 
 /* Show the [black|white]list link if we have functionality enabled. */
 $show_blacklist_link = $registry->hasMethod('mail/blacklistFrom');
@@ -472,20 +473,26 @@ if ($pageOb->msgcount != 0) {
    foreach() loop or the templates. */
 $curr_time = time();
 $curr_time -= $curr_time % 60;
+$ltime_val = localtime();
+$today_start = mktime(0, 0, 0, $ltime_val[4] + 1, $ltime_val[3], 1900 + $ltime_val[5]);
+$today_end = $today_start + 86400;
 $drafts_sm_folder = $imp_mailbox->isSpecialFolder();
 $lastMbox = '';
 $messages = array();
 $threadlevel = array();
 
 /* Determine sorting preferences. */
-$sortby = $imp_mailbox->sortby();
 $thread_sort = ($sortby == SORTTHREAD);
 $sortlimit = $imp_mailbox->aboveSortLimit();
 
 /* Get thread object, if necessary. */
 if ($thread_sort) {
     $threadob = $imp_mailbox->getThreadOb();
-    $last_level = $last_thread = null;
+    $uid_list = array();
+    foreach ($mailboxOverview as $val) {
+        $uid_list[] = $val['header']->uid;
+    }
+    $threadtree = $threadob->getThreadImageTree($uid_list, $sortdir);
 }
 
 /* Don't show header row if this is a search mailbox or if no messages in the
@@ -519,7 +526,7 @@ foreach ($mailboxOverview as $msgIndex => $message) {
 
     /* Initialize the header fields. */
     $msg = array(
-        'color' => (isset($message['color'])) ? $message['color'] : '',
+        'color' => (isset($message['color'])) ? htmlspecialchars($message['color']) : '',
         'date' => '&nbsp;',
         'from' => '&nbsp;',
         'mbox' => $message['mbox'],
@@ -540,25 +547,29 @@ foreach ($mailboxOverview as $msgIndex => $message) {
     $messages[] = $h->uid;
 
     /* Formats the header date string nicely. */
-    if (empty($h->date) || (($udate = strtotime($h->date, $curr_time)) == -1)) {
-        $msg['date'] = _("Unknown Date");
+    if (empty($h->date)) {
+        $udate = false;
     } else {
-        if ((date('Y') != @date('Y', $udate)) ||
-            (date('M') != @date('M', $udate)) ||
-            (date('d') != @date('d', $udate))) {
-            /* Not today, use the date. */
-            $msg['date'] = strftime($datefmt, $udate);
-        } else {
-            /* Else, it's today, use the time. */
-            $msg['date'] = strftime($timefmt, $udate);
-        }
+        $h->date = preg_replace('/\s+\(\w+\)$/', '', $h->date);
+        $udate = strtotime($h->date, $curr_time);
+    }
+    if ($udate === false || $udate === -1) {
+        $msg['date'] = _("Unknown Date");
+    } elseif (($udate < $today_start) || ($udate > $today_end)) {
+        /* Not today, use the date. */
+        $msg['date'] = strftime($datefmt, $udate);
+    } else {
+        /* Else, it's today, use the time. */
+        $msg['date'] = strftime($timefmt, $udate);
     }
 
     /* Format the from header. */
     if (isset($h->from)) {
         $from_adr = IMP::bareAddress($h->from);
-        $from_ob = imap_rfc822_parse_adrlist($h->from, '');
-        $from_ob = array_shift($from_ob);
+        $from_ob = IMP::parseAddressList($h->from, '');
+        if (!is_a($from_ob, 'PEAR_Error')) {
+            $from_ob = array_shift($from_ob);
+        }
         if (is_null($from_adr)) {
             $msg['from'] = _("Invalid Address");
             $showfromlink = false;
@@ -568,14 +579,19 @@ foreach ($mailboxOverview as $msgIndex => $message) {
                     $msg['from'] = _("Undisclosed Recipients");
                     $showfromlink = false;
                 } else {
-                    $tmp = imap_rfc822_parse_adrlist($h->to, '');
-                    $tmp = array_shift($tmp);
+                    $tmp = IMP::parseAddressList($h->to, '');
+                    if (!is_a($tmp, 'PEAR_Error')) {
+                        $tmp = array_shift($tmp);
+                    }
                     if (isset($tmp->personal)) {
-                        $msg['from'] = MIME::decode($tmp->personal, $charset);
+                        $msg['from'] = stripslashes(MIME::decode($tmp->personal, $charset));
                     } else {
                         $msg['from'] = IMP::bareAddress($h->to);
                     }
                     $msg['fullfrom'] = MIME::decode($h->to, $charset);
+                    if (empty($msg['from'])) {
+                        $msg['from'] = $msg['fullfrom'];
+                    }
                 }
             } else {
                 $msg['from'] = _("Undisclosed Recipients");
@@ -586,10 +602,13 @@ foreach ($mailboxOverview as $msgIndex => $message) {
             }
             $mailto_link = true;
         } elseif (isset($from_ob->personal)) {
-            $msg['from'] = trim(MIME::decode($from_ob->personal, $charset), '"');
+            $msg['from'] = trim(stripslashes(MIME::decode($from_ob->personal, $charset)), '"');
             if (!trim($msg['from'], chr(160) . ' ')) {
                 $msg['from'] = $from_adr;
             }
+            if ($drafts_sm_folder) {
+                $msg['from'] = _("From") . ': ' . $msg['from'];
+            }
             $msg['fullfrom'] = MIME::decode($h->from, $charset);
         } else {
             if (!isset($from_ob->host) ||
@@ -650,7 +669,7 @@ foreach ($mailboxOverview as $msgIndex => $message) {
 
     /* Generate the target link. */
     $msgMbox = isset($message['mbox']) ? $message['mbox'] : '';
-    $target = IMP_Search::generateSearchUrl('message.php', $msgMbox);
+    $target = IMP::generateSearchUrl('message.php', $msgMbox);
     $target = Util::addParameter($target, 'index', $h->uid);
 
     /* Get all the flag information. */
@@ -665,7 +684,6 @@ foreach ($mailboxOverview as $msgIndex => $message) {
         require_once IMP_BASE . '/lib/MIME/Headers.php';
         $imp_headers = &new IMP_Headers($h->uid);
         $imp_headers->buildHeaders();
-        $imp_headers->buildFlags();
         if (($priority = $imp_headers->getValue('x-priority'))) {
             if (preg_match('/\s*(\d+)\s*/', $priority, $matches)) {
                 if (($matches[1] == '1') || ($matches[1] == '2')) {
@@ -696,7 +714,7 @@ foreach ($mailboxOverview as $msgIndex => $message) {
         if ($h->draft) {
             $flagbits |= IMP_DRAFT;
             $msg['status'] .= Horde::img('mail_draft.png', _("Draft"));
-            $target = IMP::composeLink(array(), array('actionID' => 'draft', 'mailbox' => $imp['mailbox'], 'index' => $h->uid));
+            $target = IMP::composeLink(array(), array('actionID' => 'draft', 'mailbox' => $message['mbox'], 'index' => $h->uid));
         }
         if ($xprio == 'high') {
             $flagbits |= IMP_FLAGGED;
@@ -726,10 +744,9 @@ foreach ($mailboxOverview as $msgIndex => $message) {
     }
 
     if ($conf['mailbox']['show_attachments'] && $attachment) {
-        $msg['status'] .= Horde::img($attachment . '.png', _("Attachment"));
+        $msg['status'] .= Horde::img($attachment . '.png', $attachment_alt);
     }
 
-
     /* Show message preview? */
     if ($show_preview) {
         $msg['preview'] = $message['preview'];
@@ -751,9 +768,9 @@ foreach ($mailboxOverview as $msgIndex => $message) {
         if ($showfromlink) {
             $extra = array('actionID' => 'mailto', 'mailbox' => $imp['mailbox'], 'index' => $h->uid, 'mailto' => $mailto_link);
             if ($search_mbox) {
-                $extra['thismailbox'] = $mailboxOverview[$msgIndex]['mbox'];
+                $extra['thismailbox'] = $message['mbox'];
             }
-            $msg['from'] = Horde::link(IMP::composeLink(array(), $extra), sprintf(_("Compose Message (%s)"), $msg['fullfrom'])) . $msg['from'] . '</a>';
+            $msg['from'] = Horde::link(IMP::composeLink(array(), $extra), sprintf(_("New Message to %s"), stripslashes($msg['fullfrom']))) . $msg['from'] . '</a>';
         }
         break;
 
@@ -761,7 +778,7 @@ foreach ($mailboxOverview as $msgIndex => $message) {
         if (!isset($msg['fullfrom'])) {
             $msg['fullfrom'] = $msg['from'];
         }
-        $from_uri = IMP_Search::generateSearchUrl('message.php', $msgMbox);
+        $from_uri = IMP::generateSearchUrl('message.php', $msgMbox);
         $from_uri = Util::addParameter($from_uri, 'index', $h->uid);
         $msg['from'] = Horde::link($from_uri, $msg['fullfrom']) . $msg['from'] . '</a>';
         break;
@@ -776,38 +793,9 @@ foreach ($mailboxOverview as $msgIndex => $message) {
     }
 
     /* Set up threading tree now. */
-    if ($thread_sort && !$search_mbox) {
-        $indentBase = $threadob->getThreadBase($h->uid);
-        if (!empty($indentBase)) {
-            $indentLevel = $threadob->getThreadIndent($h->uid);
-            if ($indentLevel > 1) {
-                $lines = '';
-                $lastinlevel = $threadob->lastInLevel($h->uid);
-                if ($sortdir) {
-                    if ($indentBase != $last_thread) {
-                        $threadLevel = array();
-                        $last_thread = $indentBase;
-                        for ($i = 1; $i <= $indentLevel; $i++) {
-                            $threadLevel[$i] = true;
-                        }
-                    } elseif ($indentLevel > $last_level) {
-                        $threadLevel[$last_level] = false;
-                        for ($i = ($last_level + 1); $i <= $indentLevel; $i++) {
-                            $threadLevel[$i] = true;
-                        }
-                    }
-                    $last_level = $indentLevel;
-                    $lastinlevel = !$lastinlevel;
-                    $join_img = ($lastinlevel) ? 'join.png' : 'joinbottom-down.png';
-                } else {
-                    $join_img = ($lastinlevel) ? 'joinbottom.png' : 'join.png';
-                }
-                $threadLevel[$indentLevel] = $lastinlevel;
-                for ($i = 2; $i < $indentLevel; $i++) {
-                    $lines .= _image(!isset($threadLevel[$i]) || ($threadLevel[$i]) ? 'blank.png' : 'line.png');
-                }
-                $msg['subject'] = $lines . _image($join_img) . ' ' . $msg['subject'];
-            }
+    if ($thread_sort) {
+        if (!empty($threadtree[$h->uid])) {
+            $msg['subject'] = $threadtree[$h->uid] . ' ' . $msg['subject'];
         }
     }
 
@@ -845,15 +833,14 @@ if ($prefs->getValue('nav_popup') &&
 
 if (!empty($open_compose_window)) {
     $args = 'popup=1';
-    if (isset($options)) {
-        foreach ($options as $arg => $value) {
-            $args .= !empty($value) ? '&' . $arg . '=' . urlencode($value) : '';
-        }
+    if (!isset($options)) {
+        $options = array();
     }
-    foreach (IMP::getComposeArgs() as $arg => $value) {
+    foreach (array_merge($options, IMP::getComposeArgs()) as $arg => $value) {
         $args .= !empty($value) ? '&' . $arg . '=' . urlencode($value) : '';
     }
-    echo "<script language='JavaScript' type='text/javascript'>open_compose_win('" . addslashes($args) . "'); focus_compose_win();</script>";
+    Horde::addScriptFile('popup.js');
+    echo "<script type='text/javascript'>popup_imp('" . Horde::applicationUrl('compose.php') . "',700,650,'" . addslashes($args) . "');</script>";
 }
 
 require $registry->get('templates', 'horde') . '/common-footer.inc';
