@@ -1,9 +1,9 @@
 <?php
 /**
- * $Horde: imp/message.php,v 2.560.4.14 2005/09/26 09:39:17 selsky Exp $
+ * $Horde: imp/message.php,v 2.560.4.41 2007/01/02 13:54:54 jan Exp $
  *
- * Copyright 1999-2005 Charles J. Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2007 Charles J. Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (GPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -37,11 +37,10 @@ require_once IMP_BASE . '/lib/MIME/Contents.php';
 require_once IMP_BASE . '/lib/MIME/Headers.php';
 require_once IMP_BASE . '/lib/Mailbox.php';
 require_once IMP_BASE . '/lib/Search.php';
-require_once 'Horde/MIME.php';
 require_once 'Horde/Identity.php';
 
 /* Make sure we have a valid index. */
-$imp_mailbox = &new IMP_Mailbox(Util::getFormData('index'));
+$imp_mailbox = &IMP_Mailbox::singleton(Util::getFormData('index'));
 if (!$imp_mailbox->isValidIndex()) {
     _returnToMailbox(null, 'message_missing');
     require IMP_BASE . '/mailbox.php';
@@ -61,13 +60,15 @@ $user_identity = &Identity::singleton(array('imp', 'imp'));
 $actionID = Util::getFormData('actionID');
 switch ($actionID) {
 case 'blacklist':
-    require_once IMP_BASE . '/lib/Filter.php';
-    IMP_Filter::blacklistMessage(array($imp_mailbox->getIndex()));
-    break;
-
 case 'whitelist':
     require_once IMP_BASE . '/lib/Filter.php';
-    IMP_Filter::whitelistMessage(array($imp_mailbox->getIndex()));
+    $imp_filter = &IMP_Filter::singleton();
+    $idx = $imp_mailbox->getIMAPIndex();
+    if ($actionID == 'blacklist') {
+        $imp_filter->blacklistMessage(array($idx['mailbox'] => array($idx['index'])));
+    } else {
+        $imp_filter->whitelistMessage(array($idx['mailbox'] => array($idx['index'])));
+    }
     break;
 
 case 'print_message':
@@ -75,20 +76,19 @@ case 'print_message':
     IMP::printMode(true);
     break;
 
+case 'delete_message':
 case 'undelete_message':
     require_once IMP_BASE . '/lib/Message.php';
-    $imp_message = &new IMP_Message();
-    $imp_message->undelete($imp_mailbox);
-    break;
-
-case 'delete_message':
-    require_once IMP_BASE . '/lib/Message.php';
-    $imp_message = &new IMP_Message();
-    $imp_message->delete($imp_mailbox);
-    if ($prefs->getValue('mailbox_return')) {
-        _returnToMailbox($imp_mailbox->getMessageIndex());
-        require IMP_BASE . '/mailbox.php';
-        exit;
+    $imp_message = &IMP_Message::singleton();
+    if ($actionID == 'undelete_message') {
+        $imp_message->undelete($imp_mailbox);
+    } else {
+        $imp_message->delete($imp_mailbox);
+        if ($prefs->getValue('mailbox_return')) {
+            _returnToMailbox($imp_mailbox->getMessageIndex());
+            require IMP_BASE . '/mailbox.php';
+            exit;
+        }
     }
     break;
 
@@ -96,7 +96,7 @@ case 'move_message':
 case 'copy_message':
     if (($targetMbox = Util::getFormData('targetMbox')) !== null) {
         require_once IMP_BASE . '/lib/Message.php';
-        $imp_message = &new IMP_Message();
+        $imp_message = &IMP_Message::singleton();
 
         $action = ($actionID == 'move_message') ? IMP_MESSAGE_MOVE : IMP_MESSAGE_COPY;
 
@@ -104,11 +104,16 @@ case 'copy_message':
             (strpos($targetMbox, '_tasklist_') === 0)) {
             /* If the target is a tasklist, handle the move/copy specially. */
             $tasklist = str_replace('_tasklist_', '', $targetMbox);
-            $imp_message->createTasks($tasklist, $action, $imp_mailbox);
+            $imp_message->createTasksOrNotes($tasklist, $action, $imp_mailbox, 'task');
+        } elseif ($conf['notepad']['use_notepad'] &&
+            (strpos($targetMbox, '_notepad_') === 0)) {
+            /* If the target is a notepad, handle the move/copy specially. */
+            $notepad = str_replace('_notepad_', '', $targetMbox);
+            $imp_message->createTasksOrNotes($notepad, $action, $imp_mailbox, 'note');
         } else {
             /* Otherwise, the target is a standard mailbox. */
             if (Util::getFormData('newMbox', 0) == 1) {
-                $new_mailbox = IMP::addPreambleString(String::convertCharset($targetMbox, NLS::getCharset(), 'UTF7-IMAP'));
+                $new_mailbox = String::convertCharset(IMP::folderPref($targetMbox, true), NLS::getCharset(), 'UTF7-IMAP');
                 require_once IMP_BASE . '/lib/Folder.php';
                 $imp_folder = &IMP_Folder::singleton();
                 if ($imp_folder->create($new_mailbox, $prefs->getValue('subscribe'))) {
@@ -135,9 +140,10 @@ case 'notspam_report':
     $imp_spam->reportSpam($imp_mailbox, $action);
 
     /* Delete spam after report. */
-    if (($action == 'spam') && $prefs->getValue('delete_spam_after_report')) {
+    $delete_spam = $prefs->getValue('delete_spam_after_report');
+    if ($action == 'spam' && ($delete_spam == 1)) {
         require_once IMP_BASE . '/lib/Message.php';
-        $imp_message = &new IMP_Message();
+        $imp_message = &IMP_Message::singleton();
         $imp_message->delete($imp_mailbox);
         $notification->push(_("The message has been deleted."), 'horde.message');
         if ($prefs->getValue('mailbox_return')) {
@@ -145,6 +151,20 @@ case 'notspam_report':
             require IMP_BASE . '/mailbox.php';
             exit;
         }
+    } elseif ($delete_spam == 2) {
+        $targetMbox = ($action == 'spam') ? IMP::folderPref($prefs->getValue('spam_folder'), true) : 'INBOX';
+        if ($targetMbox) {
+            require_once IMP_BASE . '/lib/Message.php';
+            $imp_message = &IMP_Message::singleton();
+            $imp_message->copy($targetMbox, IMP_MESSAGE_MOVE, $imp_mailbox, true);
+            if ($prefs->getValue('mailbox_return')) {
+                _returnToMailbox($imp_mailbox->getMessageIndex());
+                require IMP_BASE . '/mailbox.php';
+                exit;
+            }
+        } else {
+            $notification->push(_("Could not move message to spam mailbox - no spam mailbox defined in preferences."), 'horde.error');
+        }
     }
     break;
 
@@ -159,7 +179,7 @@ case 'flag_message':
             $set = true;
         }
         require_once IMP_BASE . '/lib/Message.php';
-        $imp_message = &new IMP_Message();
+        $imp_message = &IMP_Message::singleton();
         $imp_message->flag($flag, $imp_mailbox, $set);
         if ($prefs->getValue('mailbox_return')) {
             _returnToMailbox($imp_mailbox->getMessageIndex());
@@ -181,8 +201,7 @@ case 'add_address':
 
 case 'strip_attachment':
     require_once IMP_BASE . '/lib/Message.php';
-
-    $imp_message = &new IMP_Message();
+    $imp_message = &IMP_Message::singleton();
     $result = $imp_message->stripPart($imp_mailbox, Util::getFormData('imapid'));
     if (is_a($result, 'PEAR_Error')) {
         $notification->push($result, 'horde.error');
@@ -192,7 +211,7 @@ case 'strip_attachment':
 }
 
 if ($conf['user']['allow_folders']) {
-    $options = IMP::flistSelect(_("This message to"), true, array(null, $imp['mailbox']), null, true, true);
+    $options = IMP::flistSelect(_("This message to"), true, array(), null, true, true, false, true);
 }
 
 /* We may have done processing that has taken us past the end of the
@@ -206,7 +225,9 @@ if (!$imp_mailbox->isValidIndex()) {
 
 /* Now that we are done processing the messages, get the index and
  * array index of the current message. */
-$index = $imp_mailbox->getIndex();
+$index_array = $imp_mailbox->getIMAPIndex();
+$index = $index_array['index'];
+$mailbox_name = $index_array['mailbox'];
 $array_index = $imp_mailbox->getArrayIndex();
 
 /* If we grab the headers before grabbing body parts, we'll see when a
@@ -217,8 +238,14 @@ $imp_headers->buildFlags();
 $imp_headers_copy = &Util::cloneObject($imp_headers);
 
 /* Parse MIME info and create the body of the message. */
-if (!isset($imp_contents)) {
-    $imp_contents = &IMP_Contents::singleton($index);
+$imp_contents = &IMP_Contents::singleton($index . IMP_IDX_SEP . $mailbox_name);
+
+/* Update the message flag, if necessary. */
+$use_pop = ($imp['base_protocol'] == 'pop3');
+if (!$use_pop && !$imp_headers->getFlag('seen')) {
+    require_once IMP_BASE . '/lib/Message.php';
+    $imp_message = &IMP_Message::singleton();
+    $imp_message->flag('\\SEEN', $imp_mailbox, true);
 }
 
 /* Determine if we should generate the attachment strip links or
@@ -278,9 +305,9 @@ if (($subject = $imp_headers->getValue('subject'))) {
 if (($priority = $imp_headers->getValue('x-priority'))) {
     if (preg_match("/\s*(\d+)\s*/", $priority, $matches)) {
         if (($matches[1] == '1') || ($matches[1] == '2')) {
-            $imp_headers->addHeader('Priority', Horde::img('mail_priority_high.png', _("High Priority"), 'align="absmiddle"') . '&nbsp;' . $priority);
+            $imp_headers->addHeader('Priority', Horde::img('mail_priority_high.png', _("High Priority")) . '&nbsp;' . $priority);
         } elseif (($matches[1] == '4') || ($matches[1] == '5')) {
-            $imp_headers->addHeader('Priority', Horde::img('mail_priority_low.png', _("Low Priority"), 'align="absmiddle"') . '&nbsp;' . $priority);
+            $imp_headers->addHeader('Priority', Horde::img('mail_priority_low.png', _("Low Priority")) . '&nbsp;' . $priority);
         }
     }
 }
@@ -323,22 +350,26 @@ if (!empty($user_hdrs) && !$all_headers) {
     }
 }
 
+/* Generate the list of search parameters. */
+$search_params = IMP::getSearchParameters($mailbox_name, $index);
+
 /* For the self URL link, we can't trust the index in the query string as it
  * may have changed if we deleted/copied/moved messages. We may need other
  * stuff in the query string, so we need to do an add/remove of 'index'. */
 $selfURL = Util::removeParameter(Horde::selfUrl(true), array('index', 'actionID'));
 $selfURL = Util::addParameter($selfURL, 'index', $index);
-$headersURL = Util::removeParameter($selfURL, array('show_all_headers', 'show_list_headers'));
+$headersURL = Util::removeParameter($selfURL, array('show_all_headers', 'show_list_headers', 'mailbox', 'thismailbox'));
+$headersURL = Util::addParameter($headersURL, $search_params);
 
 /* Determine previous message index. */
-if (($prev_msg = $imp_mailbox->messageIndices(-1))) {
-    $prev_url = IMP_Search::generateSearchUrl('message.php', $prev_msg['mailbox']);
+if (($prev_msg = $imp_mailbox->getIMAPIndex(-1))) {
+    $prev_url = IMP::generateSearchUrl('message.php', $prev_msg['mailbox']);
     $prev_url = Util::addParameter($prev_url, 'index', $prev_msg['index']);
 }
 
 /* Determine next message index. */
-if (($next_msg = $imp_mailbox->messageIndices(1))) {
-    $next_url = IMP_Search::generateSearchUrl('message.php', $next_msg['mailbox']);
+if (($next_msg = $imp_mailbox->getIMAPIndex(1))) {
+    $next_url = IMP::generateSearchUrl('message.php', $next_msg['mailbox']);
     $next_url = Util::addParameter($next_url, 'index', $next_msg['index']);
 }
 
@@ -350,47 +381,38 @@ $msgcount = $imp_mailbox->getMessageCount();
 /* Generate the mailbox link. */
 $mailbox_url = Util::addParameter(Horde::applicationUrl('mailbox.php'), 'start', $msgindex);
 
-/* Get the current mailbox name. */
-$mailbox_name = IMP::getThisMailbox($array_index);
-
 /* Generate the view link. */
-$view_link = IMP_Search::generateSearchUrl('view.php', $mailbox_name);
+$view_link = IMP::generateSearchUrl('view.php', $mailbox_name);
 $view_link = Util::addParameter($view_link, 'index', $index);
 
 /* Generate the Save Message link. */
-$search_params = IMP_Search::getSearchParameters($mailbox_name);
 $save_link_array = array_merge(array('actionID' => 'save_message', 'index' => $index), $search_params);
 $save_link = Horde::downloadUrl($subject, $save_link_array);
 
 /* Generate the Message Source link. */
-if ($conf['user']['allow_view_source']) {
+if (!empty($conf['user']['allow_view_source'])) {
     $base_part = $imp_contents->getMIMEMessage();
     $source_link = $imp_contents->linkViewJS($base_part, 'view_source', _("_Message Source"), _("Message Source"), 'widget', array(), true);
 }
 
 /* Generate the link to ourselves. */
-$self_link = IMP_Search::generateSearchUrl('message.php', $mailbox_name);
+$self_link = IMP::generateSearchUrl('message.php', $mailbox_name);
 $self_link = Util::addParameter($self_link, array('index' => $index, 'start' => $msgindex));
 
 /* Generate the print link. */
-$print_link = '';
 $print_params = array('actionID' => 'print_message', 'index' => $index);
-foreach (array_merge($print_params, $search_params) as $key => $val) {
-    $print_link = Util::addParameter($print_link, $key, $val);
-}
-if (substr($print_link, 0, 1) == '?') {
-    $print_link = substr($print_link, 1);
-}
-$print_link = $browser->escapeJSCode($print_link);
+$print_link = Horde::applicationUrl('message.php');
+$print_link = Util::addParameter($print_link, array_merge($print_params, $search_params));
 
 /* Generate the thread view link. */
 $thread_link = Util::addParameter(Horde::applicationUrl('thread.php'), array('index' => $index, 'start' => $msgindex));
 
 $delete_warning = '';
-if ($imp['base_protocol'] == 'pop3') {
+if ($use_pop) {
     $delete_warning = "return window.confirm('" . addslashes(_("Are you sure you wish to PERMANENTLY delete these messages?")) . "');";
 }
 
+Horde::addScriptFile('popup.js', 'horde', true);
 Horde::addScriptFile('keybindings.js', 'horde');
 require IMP_TEMPLATES . '/common-header.inc';
 
@@ -455,7 +477,7 @@ foreach ($address_headers as $key => $val) {
 }
 
 /* Build Reply-To address links. */
-if (($reply_to = $imp_headers->buildAddressLinks('reply_to', $self_link, false, !$printer_friendly))) {
+if (($reply_to = $imp_headers->buildAddressLinks('reply-to', $self_link, false, !$printer_friendly))) {
     if (!($from = $imp_headers->getValue('from')) || ($from != $reply_to)) {
         $imp_headers->setValue('Reply-to', $reply_to);
     } else {
@@ -467,15 +489,16 @@ if (($reply_to = $imp_headers->buildAddressLinks('reply_to', $self_link, false,
 $addresses = array_keys($user_identity->getAllFromAddresses(true));
 $identity = null;
 $status = '';
-if ($imp['base_protocol'] != 'pop3') {
+if (!$use_pop) {
     if (count($msgAddresses)) {
         $identity = $user_identity->getMatchingIdentity($msgAddresses);
+        if (!is_null($identity) ||
+            $user_identity->getMatchingIdentity($msgAddresses, false) !== null) {
+            $status .= Horde::img('mail_personal.png', _("Personal"));
+        }
         if (is_null($identity)) {
             $identity = $user_identity->getDefault();
         }
-        if ($user_identity->getMatchingIdentity($msgAddresses, false) !== null) {
-            $status .= Horde::img('mail_personal.png', _("Personal"));
-        }
     }
 
     /* Set status flags. */
@@ -555,10 +578,6 @@ if (!$printer_friendly) {
         }
     }
 
-    if ($browser->hasFeature('javascript')) {
-        require_once IMP_TEMPLATES . '/javascript/open_print_win.js';
-    }
-
     require IMP_TEMPLATES . '/message/navbar_top.inc';
     $nav_id = 1;
     require IMP_TEMPLATES . '/message/navbar_navigate.inc';
@@ -568,9 +587,7 @@ if (!$printer_friendly) {
     require IMP_TEMPLATES . '/message/navbar_actions.inc';
     $actions_output = ob_get_contents();
     ob_end_clean();
-    echo $actions_output;
-
-    echo '</table>';
+    echo $actions_output . '</table>';
 }
 
 /* Generate some variables needed for the templates. */
@@ -582,8 +599,7 @@ require IMP_TEMPLATES . '/message/headers.inc';
 require IMP_TEMPLATES . '/message/message.inc';
 
 if (!$printer_friendly) {
-    echo '<table border="0" align="center" width="100%" cellspacing="0" cellpadding="2">';
-    echo $actions_output;
+    echo '<table width="100%" cellspacing="0">' . $actions_output;
     $nav_id = 2;
     require IMP_TEMPLATES . '/message/navbar_navigate.inc';
     echo '</table></form>';
